import { h, Host, getElement } from "@stencil/core";
import { getStyles } from "./getStyle";
const useAdoptedStyles = useAdoptedStyleSheets();
export function CssInJs(options) {
    return (target, propertyKey) => {
        const componentName = target.constructor.name;
        const originalWillLoad = target.componentWillLoad;
        if (!originalWillLoad) {
            console.error(`componentWillLoad lifecycle method in ${componentName} missing. Required for CssInJs decorator.`);
        }
        if (useAdoptedStyles) {
            target.componentWillLoad = function () {
                var _a, _b;
                const willLoadResult = (_a = originalWillLoad) === null || _a === void 0 ? void 0 : _a.call(this);
                const host = getElement(this);
                const root = (_b = host.shadowRoot, (_b !== null && _b !== void 0 ? _b : host));
                const stylesObject = getStyleObject(this, options.styles);
                watchTheme(this, options.styles, (newStyleObject) => {
                    const adoptedSheet = root.adoptedStyleSheets;
                    const index = adoptedSheet.findIndex(sheet => sheet["data-cssInJsId"]);
                    if (index !== -1) {
                        root.adoptedStyleSheets = Object.assign([], adoptedSheet, {
                            [index]: newStyleObject.adoptedSheet
                        });
                    }
                    else {
                        root.adoptedStyleSheets = [
                            ...adoptedSheet,
                            newStyleObject.adoptedSheet
                        ];
                    }
                    host.forceUpdate();
                });
                root.adoptedStyleSheets = [
                    ...root.adoptedStyleSheets,
                    stylesObject.adoptedSheet
                ];
                return willLoadResult;
            };
            target.render = function () {
                return this[propertyKey](getStyleObject(this, options.styles).sheet.classes);
            };
        }
        else {
            target.componentWillLoad = function () {
                var _a;
                const willLoadResult = (_a = originalWillLoad) === null || _a === void 0 ? void 0 : _a.call(this);
                watchTheme(this, options.styles, () => {
                    getElement(this).forceUpdate();
                });
                return willLoadResult;
            };
            target.render = function () {
                const stylesObject = getStyleObject(this, options.styles);
                const cssString = stylesObject.sheet.toString();
                let renderResult = this[propertyKey](stylesObject.sheet.classes);
                if (isHost(renderResult)) {
                    prependStyleNode(renderResult, componentName, cssString);
                }
                else {
                    renderResult = h(Host, null, renderResult);
                    prependStyleNode(renderResult, componentName, cssString);
                }
                return renderResult;
            };
        }
    };
}
function useAdoptedStyleSheets() {
    return Boolean(document.adoptedStyleSheets);
}
function prependStyleNode(node, componentName, cssString) {
    var _a;
    (_a = node["$children$"], (_a !== null && _a !== void 0 ? _a : [])).unshift(h("style", { type: "text/css", "css-in-js": componentName }, cssString));
}
function isHost(node) {
    return Object.values(node).includes(Host);
}
function getStyleObject(instance, styles) {
    return getStyles({
        theme: instance.theme,
        styles: styles,
        componentName: instance.constructor.name,
        useAdoptedStyles
    });
}
function watchTheme(instance, styles, callback) {
    if (instance.theme) {
        let theme = instance.theme;
        Object.defineProperty(instance, "theme", {
            get: () => {
                return theme;
            },
            set: newValue => {
                var _a;
                theme = newValue;
                let themeObject = getStyles({
                    theme: newValue,
                    styles,
                    componentName: instance.constructor.name,
                    useAdoptedStyles
                });
                (_a = callback) === null || _a === void 0 ? void 0 : _a(themeObject);
            }
        });
    }
}
